function CVPR15Feature(LIB, VS, PATH, Param, LabelSet, Flag)


%% Texton Filter Bank Response

if Flag.feature.mr
    
    if ~exist(PATH.ftMrBook, 'file')
        fprintf('Compute MR Filter Bank Codebook ...\n');
        fpath_list{length(VS.vid)} = [];
        for i=1:length(VS.vid)
            fpath_list{i} = fullfile(VS.pngPath, VS.vid{i});
        end
        ftMrBook(LIB.Filters, fpath_list, PATH.ftMrBook, Param.feature.mr);
    end
    
    fprintf('Compute MR Filter Bank for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(VS.pngPath, VS.vid{i});
        savepath = fullfile(PATH.ftMr, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            ftMr(LIB.Filters, PATH.ftMrBook, ...
                fpath, savepath, Param.feature.mr);
        end
    end
    
end

%% Dense SIFT Feature

if Flag.feature.sift
    
    if ~exist(PATH.ftSiftBook, 'file')
        fprintf('Compute Dense SIFT Codebook ...\n');
        fpath_list{length(VS.vid)} = [];
        for i=1:length(VS.vid)
            fpath_list{i} = fullfile(VS.pngPath, VS.vid{i});
        end
        ftSiftBook(LIB.SpatialPyramid, fpath_list, ...
            PATH.ftSiftBook, Param.feature.sift);
    end
    
    fprintf('Compute Dense SIFT for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(VS.pngPath, VS.vid{i});
        savepath = fullfile(PATH.ftSift, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            ftSift(LIB.SpatialPyramid, PATH.ftSiftBook, ...
                fpath, savepath, Param.feature.sift);
        end
    end
    
end

%% Optical Flow

if Flag.feature.flow
    
    % Compute raw optical flow.
    fprintf('Compute Optical Flow for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(VS.pngPath, VS.vid{i});
        savefold = fullfile(PATH.ftFlowRaw, VS.vid{i});
        ftFlowRaw(LIB.Flow, fpath, savefold);
    end

    % Move TSP flows to raw flow
%     for i=1:length(VS.vid)
%         old_path = fullfile(VS.pngPath, VS.vid{i}, 'TSP_flows');
%         new_path = fullfile(PATH.ftFlowRaw, VS.vid{i});
%         cmd = sprintf('mv %s %s', old_path, new_path);
%         system(cmd);
%     end
    
    % Sample Codebook Centers.
    if ~exist(PATH.ftFlowBook, 'file')
        fprintf('Compute Optical Flow Codebook ...\n');
        fpath_list{length(VS.vid)} = [];
        for i=1:length(VS.vid)
            fpath_list{i} = fullfile(PATH.ftFlowRaw, VS.vid{i});
        end
        ftFlowBook([], fpath_list(1:10:end), ...
            PATH.ftFlowBook, Param.feature.flow);
    end
    
    % Encode all Videos.
    fprintf('Encoding Optical Flow for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(PATH.ftFlowRaw, VS.vid{i});
        savepath = fullfile(PATH.ftFlow, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            ftFlow([], PATH.ftFlowBook, fpath, ...
                savepath, Param.feature.flow);
        end
    end
    
end


%% Dense Trajectories

if Flag.feature.dt
    
    fprintf('Compute Dense Trajectory Raw Features ...\n');
    if ~exist(PATH.ftDtRaw, 'dir')
        mkdir(PATH.ftDtRaw);
    end
    parfor i=1:length(VS.vid)
        video_path = fullfile(VS.clipPath, [VS.vid{i},'.avi']);
        txt_path = fullfile(PATH.ftDtRaw, [VS.vid{i},'.txt']);
        mat_path = fullfile(PATH.ftDtRaw, [VS.vid{i},'.mat']);
        if ~exist(mat_path, 'file')
            cmd = sprintf('%s %s > %s', LIB.DT, video_path, txt_path);
            system(cmd);
            text2mat(txt_path, mat_path);
        end
    end
    
    if ~exist(PATH.ftDtBook, 'file')
        fprintf('Compute Dense Trajectory Codebook ...\n');
        fpath_list{length(VS.vid)} = [];
        for i=1:length(VS.vid)
            fpath_list{i} = fullfile(PATH.ftDtRaw, [VS.vid{i},'.mat']);
        end
        ftDtBook([], fpath_list(1:10:end), ...
            PATH.ftDtBook, Param.feature.dt);
    end
    
    fprintf('Encode Dense Trajectory for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(PATH.ftDtRaw, [VS.vid{i},'.mat']);
        savepath = fullfile(PATH.ftDt, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            ftDt([], PATH.ftDtBook, fpath, savepath, Param.feature.dt);
        end
    end
    
end

