function CVPR15Model(LIB, VS, PATH, Param, LabelSet, Flag)


%% UGM CVPR'15 Models

%% Flat: Naive Bayes

if Flag.model.naivebayes
    
    % Obj
    
    w = [Param.model.flat.obj.alpha; ...
        Param.model.flat.obj.theta];
            
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            labelMap = (1:LabelSet.objNum+1)';
            
            svBK = load(fullfile(PATH.svTSPPotn,'BK', ...
                [VS.vid{i},'.mat']));
            svPot = load(fullfile(PATH.svTSPPotn,'Obj', ...
                [VS.vid{i},'.mat']));
            unary = [svBK.potential, svPot.potential];
            
            svFv = load(fullfile(PATH.svTSPDesc,'base', ...
                [VS.vid{i},'.mat']));
            svBmap = load(fullfile(PATH.svTSPAffy,[VS.vid{i},'.mat']));
            Bmap = svBmap.Bmap;
            Bvalue = setBvalue_UGM(Bmap, svFv.fv, w(2));
            
            savepath = fullfile(PATH.modFlat,'Obj',[VS.vid{i},'.mat']);
            flatInfer_UGM(LIB.UGM, labelMap, unary, Bmap, Bvalue, ...
                savepath, w);
            
            toc;
        end
    end
    
    
    % Act
    
    w = [Param.model.flat.act.alpha; ...
        Param.model.flat.act.theta];
    
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            labelMap = (1:LabelSet.actNum+1)';
            
            svBK = load(fullfile(PATH.svTSPPotn,'BK', ...
                [VS.vid{i},'.mat']));
            svPot = load(fullfile(PATH.svTSPPotn,'Act', ...
                [VS.vid{i},'.mat']));
            unary = [svBK.potential, svPot.potential];
            
            svFv = load(fullfile(PATH.svTSPDesc,'base', ...
                [VS.vid{i},'.mat']));
            svBmap = load(fullfile(PATH.svTSPAffy,[VS.vid{i},'.mat']));
            Bmap = svBmap.Bmap;
            Bvalue = setBvalue_UGM(Bmap, svFv.fv, w(2));
            
            savepath = fullfile(PATH.modFlat,'Act',[VS.vid{i},'.mat']);
            flatInfer_UGM(LIB.UGM, labelMap, unary, Bmap, Bvalue, ...
                savepath, w);
            
            toc;
        end
    end
    
    if ~exist(fullfile(PATH.modFlat,'combine'),'dir')
        mkdir(fullfile(PATH.modFlat,'combine'));
    end
    
    for i=1:length(VS.vid)
        if VS.test(i) == 1
            
            objLabel = load(fullfile(PATH.modFlat,'Obj', ...
                [VS.vid{i},'.mat']));
            actLabel = load(fullfile(PATH.modFlat,'Act', ...
                [VS.vid{i},'.mat']));
            
            labelIDX = (objLabel.labelIDX-1)*10 + (actLabel.labelIDX-1);
            labelIDX(labelIDX<10) = 0;
            labelIDX(labelIDX==0) = 1;
            
            savepath = fullfile(PATH.modFlat,'combine',[VS.vid{i},'.mat']);
            save(savepath, 'labelIDX', '-v7.3');
        end
    end
    
end

%% Flat: JointPS

if Flag.model.jointps
    
    w = [Param.model.flat.objact.alpha; ...
        Param.model.flat.objact.theta];
    
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            labelMap = [1; LabelSet.allSet];
            
            svBK = load(fullfile(PATH.svTSPPotn,'BK', ...
                [VS.vid{i},'.mat']));
            svPot = load(fullfile(PATH.svTSPPotn,'ObjAct', ...
                [VS.vid{i},'.mat']));
            unary = [svBK.potential, svPot.potential];
            
            svFv = load(fullfile(PATH.svTSPDesc,'base', ...
                [VS.vid{i},'.mat']));
            svBmap = load(fullfile(PATH.svTSPAffy,[VS.vid{i},'.mat']));
            Bmap = svBmap.Bmap;
            Bvalue = setBvalue_UGM(Bmap, svFv.fv, w(2));
            
            savepath = fullfile(PATH.modFlat,'ObjAct',[VS.vid{i},'.mat']);
            flatInfer_UGM(LIB.UGM, labelMap, unary, Bmap, Bvalue, ...
                savepath, w);
            
            toc;
        end
    end
    
end


%% Conditional Model

if Flag.model.conditional
    
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            svMap = load(fullfile(PATH.svTSPMat,[VS.vid{i},'.mat']));
            labelIDX_Obj = load(fullfile(PATH.modFlat,'Obj', ...
                [VS.vid{i},'.mat']));
            svPotActOnObj = load(fullfile(PATH.svTSPPotn,'ActOnObj', ...
                [VS.vid{i},'.mat']));
            
            svMap = svMap.svMap;
            objNum = LabelSet.objNum;
            actNum = LabelSet.actNum;
            
            savepath = fullfile(PATH.modCondition, [VS.vid{i},'.mat']);
            conInfer(svMap, objNum, labelIDX_Obj.labelIDX, ...
                actNum, svPotActOnObj.potential, savepath);
            
            toc;
        end
    end
    
end


%% Bilayer Model

if Flag.model.bilayer
        
    w = [Param.model.bilayer.obj.alpha; ...
        Param.model.bilayer.obj.theta; ...
        Param.model.bilayer.act.alpha; ...
        Param.model.bilayer.act.theta; ...
        Param.model.bilayer.obj.weight; ...
        Param.model.bilayer.objact.weight; ...
        Param.model.bilayer.act.weight];
    
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            svBK = load(fullfile(PATH.svTSPPotn,'BK', ...
                [VS.vid{i},'.mat']));
            svPotObj = load(fullfile(PATH.svTSPPotn,'Obj', ...
                [VS.vid{i},'.mat']));
            svPotAct = load(fullfile(PATH.svTSPPotn,'Act', ...
                [VS.vid{i},'.mat']));
            svPotObjAct = load(fullfile(PATH.svTSPPotn,'ObjAct', ...
                [VS.vid{i},'.mat']));
            svFv = load(fullfile(PATH.svTSPDesc,'base', ...
                [VS.vid{i},'.mat']));
            svBmap = load(fullfile(PATH.svTSPAffy,[VS.vid{i},'.mat']));
            
            labelMap_upper = (1:LabelSet.objNum+1)';
            unary_upper = [svBK.potential, svPotObj.potential];
            Bmap_upper = svBmap.Bmap;
            Bvalue_upper = setBvalue_UGM(Bmap_upper, svFv.fv, w(2));
            
            labelMap_lower = (1:LabelSet.actNum+1)';
            unary_lower = [svBK.potential, svPotAct.potential];
            Bmap_lower = svBmap.Bmap;
            Bvalue_lower = setBvalue_UGM(Bmap_lower, svFv.fv, w(4));
            
            num_nodes = size(unary_upper,1);
            num_states_upper = length(labelMap_upper);
            num_states_lower = length(labelMap_lower);
            link_l2u = zeros(num_states_upper, ...
                num_states_lower, num_nodes);
            for k=1:num_nodes
                Pot_mat = reshape(svPotObjAct.potential(k,:), ...
                    LabelSet.actNum, LabelSet.objNum);
                Pot_mat = Pot_mat';
                Pot_mat = cat(1, zeros(1,LabelSet.actNum), Pot_mat);
                Pot_mat = cat(2, [svBK.potential(k); ...
                    zeros(LabelSet.objNum,1)], Pot_mat);
                link_l2u(:,:,k) = Pot_mat;
            end
            
            savepath = fullfile(PATH.modBilayer,[VS.vid{i},'.mat']);
            biInfer_UGM(LIB.UGM, ...
                labelMap_upper, unary_upper, Bmap_upper, Bvalue_upper, ...
                labelMap_lower, unary_lower, Bmap_lower, Bvalue_lower, ...
                link_l2u, savepath, w);
            
            toc;
        end
    end
    
end


%% Trilayer Model

if Flag.model.trilayer
        
    w = [Param.model.trilayer.objact.alpha; ...
        Param.model.trilayer.objact.theta; ...
        Param.model.trilayer.obj.alpha; ...
        Param.model.trilayer.obj.theta; ...
        Param.model.trilayer.act.alpha; ...
        Param.model.trilayer.act.theta; ...
        Param.model.trilayer.objact2obj.weight; ...
        Param.model.trilayer.objact2act.weight; ...
        Param.model.trilayer.objact.weight; ...
        Param.model.trilayer.obj.weight; ...
        Param.model.trilayer.act.weight];
    
    parfor i=1:length(VS.vid)
        if VS.test(i) == 1
            tic;
            
            svBK = load(fullfile(PATH.svTSPPotn,'BK', ...
                [VS.vid{i},'.mat']));
            svPotObj = load(fullfile(PATH.svTSPPotn,'Obj', ...
                [VS.vid{i},'.mat']));
            svPotAct = load(fullfile(PATH.svTSPPotn,'Act', ...
                [VS.vid{i},'.mat']));
            svPotObjAct = load(fullfile(PATH.svTSPPotn,'ObjAct', ...
                [VS.vid{i},'.mat']));
            svPotActOnObj = load(fullfile(PATH.svTSPPotn,'ActOnObj', ...
                [VS.vid{i},'.mat']));
            svPotObjOnAct = load(fullfile(PATH.svTSPPotn,'ObjOnAct', ...
                [VS.vid{i},'.mat']));
            svFv = load(fullfile(PATH.svTSPDesc,'base', ...
                [VS.vid{i},'.mat']));
            svBmap = load(fullfile(PATH.svTSPAffy,[VS.vid{i},'.mat']));
            
            labelMap_middle = [1;LabelSet.allSet];
            unary_middle = [svBK.potential, svPotObjAct.potential];
            Bmap_middle = svBmap.Bmap;
            Bvalue_middle = setBvalue_UGM(Bmap_middle, svFv.fv, w(2));
            
            labelMap_upper = (1:LabelSet.objNum+1)';
            unary_upper = [svBK.potential, svPotObj.potential];
            Bmap_upper = svBmap.Bmap;
            Bvalue_upper = setBvalue_UGM(Bmap_upper, svFv.fv, w(4));
            
            labelMap_lower = (1:LabelSet.actNum+1)';
            unary_lower = [svBK.potential, svPotAct.potential];
            Bmap_lower = svBmap.Bmap;
            Bvalue_lower = setBvalue_UGM(Bmap_lower, svFv.fv, w(6));
            
            num_nodes = size(unary_middle,1);
            num_states_middle = length(labelMap_middle);
            num_states_upper = length(labelMap_upper);
            num_states_lower = length(labelMap_lower);
            
            link_m2u = zeros(num_states_middle,...
                num_states_upper, num_nodes);
            for k=1:num_nodes
                Pot_mat = zeros(LabelSet.allSetNum, LabelSet.objNum);
                for iter=1:LabelSet.objNum
                    con_startPos = (iter-1)*LabelSet.actNum+1;
                    con_endPos = iter*LabelSet.actNum;
                    pot_startPos = (iter-1)*LabelSet.actNum+1;
                    pot_endPos = iter*LabelSet.actNum;
                    Pot_mat(pot_startPos:pot_endPos,iter) = ...
                        svPotActOnObj.potential(k,con_startPos:con_endPos);
                end
                Pot_mat = cat(1, zeros(1,LabelSet.objNum), Pot_mat);
                Pot_mat = cat(2, [svBK.potential(k); ...
                    zeros(LabelSet.allSetNum,1)], Pot_mat);
                link_m2u(:,:,k) = Pot_mat;
            end
            
            link_m2l = zeros(num_states_middle, ...
                num_states_lower, num_nodes);
            for k=1:num_nodes
                Pot_mat = zeros(LabelSet.allSetNum, LabelSet.actNum);
                for iter=1:LabelSet.objNum
                    con_startPos = (iter-1)*LabelSet.actNum+1;
                    con_endPos = iter*LabelSet.actNum;
                    ObjOnAct_thisone = svPotObjOnAct.potential(k, ...
                        con_startPos:con_endPos);
                    pot_startPos = (iter-1)*LabelSet.actNum+1;
                    pot_endPos = iter*LabelSet.actNum;
                    Pot_mat(pot_startPos:pot_endPos,:) = ...
                        repmat(ObjOnAct_thisone,LabelSet.actNum,1).* ...
                        eye(LabelSet.actNum);
                end
                Pot_mat = cat(1, zeros(1,LabelSet.actNum), Pot_mat);
                Pot_mat = cat(2, [svBK.potential(k); ...
                    zeros(LabelSet.allSetNum,1)], Pot_mat);
                link_m2l(:,:,k) = Pot_mat;
            end
            
            savepath = fullfile(PATH.modTrilayer, [VS.vid{i},'.mat']);
            triInfer_UGM(LIB.UGM, ...
                labelMap_middle, unary_middle, Bmap_middle, Bvalue_middle, ...
                labelMap_upper, unary_upper, Bmap_upper, Bvalue_upper, ...
                labelMap_lower, unary_lower, Bmap_lower, Bvalue_lower, ...
                link_m2u, link_m2l, savepath, w);
            
            toc;
        end
    end
    
end













