function CVPR15Segment(LIB, VS, PATH, Param, LabelSet, Flag)


%% Generate Supervoxels.

if Flag.segment.tsp
    
    fprintf('Compute Segmentations for all videos ...\n');
    parfor i=1:length(VS.vid)
        fpath = fullfile(VS.pngPath, VS.vid{i});
        savepath = fullfile(PATH.svTSPMat, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            svTSP(LIB.TSP, fpath, 'png', savepath, Param.segment.tsp);
        end
    end
    
end

%% Supervoxel Features.

if Flag.segment.descriptor
    
    fprintf('Compute Segment Descriptors for all videos ...\n');
    
    parfor i=1:length(VS.vid)
        frame_path = fullfile(VS.pngPath, VS.vid{i});
        frame_fmt = 'png';
        svMap_path = fullfile(PATH.svTSPMat, [VS.vid{i},'.mat']);
        ftMr_path = fullfile(PATH.ftMr, [VS.vid{i},'.mat']);
        ftSift_path = fullfile(PATH.ftSift, [VS.vid{i},'.mat']);
        ftFlow_path = fullfile(PATH.ftFlow, [VS.vid{i},'.mat']);
        savepath = fullfile(PATH.svTSPDesc, 'base', [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            svDescBase(frame_path, frame_fmt, svMap_path, ftMr_path, ...
                ftSift_path, ftFlow_path, savepath, Param.feature);
        end
    end
    
    parfor i=1:length(VS.vid)
        svMap_path = fullfile(PATH.svTSPMat, [VS.vid{i},'.mat']);
        ftDt_path = fullfile(PATH.ftDt, [VS.vid{i},'.mat']);
        savepath = fullfile(PATH.svTSPDesc, 'dt', [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            svDescDT(svMap_path, ftDt_path, savepath, Param.feature.dt);
        end
    end
    
end

%% Supervoxel Affinity.

if Flag.segment.affinity
        
    fprintf('Compute Segments Affinity for all videos ...\n')
    parfor i=1:length(VS.vid)
        svMap_path = fullfile(PATH.svTSPMat, [VS.vid{i},'.mat']);
        savepath = fullfile(PATH.svTSPAffy, [VS.vid{i},'.mat']);
        if ~exist(savepath, 'file')
            svAffinity(svMap_path, savepath);
        end
    end
    
end

%% Supervoxel Ground Truth
if Flag.segment.groundtruth
    
    if ~exist(PATH.svTSPGTmat, 'file')
        fprintf('Load ground-truth labels to segments ... \n');
        segLabels = cell(length(VS.vid),1);
        labelList = cell(length(VS.vid),1);
        segSizes = cell(length(VS.vid),1);
        parfor i=1:length(VS.vid)
            fprintf('processing %d ...\n', i);
            svMap_path = fullfile(PATH.svTSPMat, [VS.vid{i},'.mat']);
            gt_path = fullfile(VS.gtPixIdPath, VS.vid{i});
            [segLabels{i}, labelList{i}, segSizes{i}] = ...
                svGroundTruth(svMap_path, gt_path);
        end
        save(PATH.svTSPGTmat, 'segLabels', 'labelList', ...
            'segSizes', '-v7.3');
    end
    
end


%% This module is used to train segment classifiers

if Flag.segment.classifier
    
    % Load segment ground truth
    SB = load(PATH.svTSPGTmat);
    
    % Dataset Video Statistics
    poolObjAct = [];
    numObjAct = zeros(length(VS.vid),1);
    numObj = zeros(length(VS.vid),1);
    numAct = zeros(length(VS.vid),1);
    for i=1:length(VS.vid)
        thisLabelList = SB.labelList{i};
        if length(thisLabelList) > 1
            thisLabelList = thisLabelList(thisLabelList>1);
            poolObjAct = [poolObjAct; thisLabelList];
            numObjAct(i) = length(thisLabelList);
            numObj(i) = length(unique(floor(thisLabelList/10)));
            numAct(i) = length(unique(mod(thisLabelList,10)));
        else
            fprintf('video %s contains no pixel-level annotation.\n', ...
                VS.vid{i});
            numObjAct(i) = 1; numObj(i) = 1; numAct(i) = 1;
        end
    end
    [uniques, numUnique] = count_unique(poolObjAct);
    for i=1:length(uniques)
        fprintf('label: %d -- num of video: %d \n', ...
            uniques(i), numUnique(i));
    end
    
    % Dataset Segments Statistics
    poolObjAct = [];
    for i=1:length(VS.vid)
        poolObjAct = [poolObjAct; SB.segLabels{i}];
    end
    poolObjAct = poolObjAct(poolObjAct>1);
    [~,numUnique] = count_unique(poolObjAct);
    figure; bar(numUnique); title('ObjAct Segment Numbers (Train & Test)');
    poolObj = floor(poolObjAct/10);
    [~,numUnique] = count_unique(poolObj);
    figure; bar(numUnique); title('Obj Segment Numbers (Train & Test)');
    poolAct = mod(poolObjAct,10);
    [~,numUnique] = count_unique(poolAct);
    figure; bar(numUnique); title('Act Segment Numbers (Train & Test)');
    
    
    % Training
    paramList = [];
    for i=1:LabelSet.actNum
        paramList(end+1).funcName = 'svClassifiers';
        paramList(end).labelID = LabelSet.act(i);
        paramList(end).classType = 'Act';
        paramList(end).trainFlag = 1;
        paramList(end).savepath = fullfile(PATH.svTSPClsf, 'Act', ...
            sprintf('%02d.mat',LabelSet.act(i)));
    end
    for i=1:LabelSet.objNum
        paramList(end+1).funcName = 'svClassifiers';
        paramList(end).labelID = LabelSet.obj(i);
        paramList(end).classType = 'Obj';
        paramList(end).trainFlag = 1;
        paramList(end).savepath = fullfile(PATH.svTSPClsf, 'Obj', ...
            sprintf('%02d.mat',LabelSet.obj(i)));
    end
    for i=1:LabelSet.jointNum
        paramList(end+1).funcName = 'svClassifiers';
        paramList(end).labelID = LabelSet.joint(i);
        paramList(end).classType = 'ObjAct';
        paramList(end).trainFlag = 1;
        paramList(end).savepath = fullfile(PATH.svTSPClsf, 'ObjAct', ...
            sprintf('%02d.mat',LabelSet.joint(i)));
    end
    paramList(end+1).funcName = 'svClassifiersBK';
    paramList(end).trainFlag = 1;
    paramList(end).savepath = fullfile(PATH.svTSPClsf, 'BK', 'BK.mat');
    for i=1:LabelSet.objNum
        for j=1:LabelSet.actNum
            paramList(end+1).funcName = 'svClassifiersActOnObj';
            paramList(end).ActLabel = j;
            paramList(end).ObjLabel = i;
            paramList(end).trainFlag = 1;
            paramList(end).savepath = fullfile(PATH.svTSPClsf, ...
                'ActOnObj', sprintf('%02d.mat',i*10+j));
        end
    end
    for i=1:LabelSet.actNum
        for j=1:LabelSet.objNum
            paramList(end+1).funcName = 'svClassifiersObjOnAct';
            paramList(end).ObjLabel = j;
            paramList(end).ActLabel = i;
            paramList(end).trainFlag = 1;
            paramList(end).savepath = fullfile(PATH.svTSPClsf, ...
                'ObjOnAct', sprintf('%02d.mat',j*10+i));
        end
    end

    fprintf('Training Segment Classifiers ... \n')
    parfor i=1:length(paramList)
        if ~exist(paramList(i).savepath, 'file')
            if strcmp(paramList(i).funcName, 'svClassifiers')
                svClassifiers(LIB.Libsvm, VS, PATH, LabelSet, SB, ...
                    paramList(i).labelID, paramList(i).classType, ...
                    paramList(i).trainFlag, paramList(i).savepath);
            end
            if strcmp(paramList(i).funcName, 'svClassifiersBK')
                svClassifiersBK(LIB.Libsvm, VS, PATH, LabelSet, SB, ...
                    paramList(i).trainFlag, paramList(i).savepath);
            end
            if strcmp(paramList(i).funcName, 'svClassifiersActOnObj')
                svClassifiersActOnObj(LIB.Libsvm, VS, PATH, LabelSet, ...
                    SB, paramList(i).ActLabel, paramList(i).ObjLabel, ...
                    paramList(i).trainFlag, paramList(i).savepath);
            end
            if strcmp(paramList(i).funcName, 'svClassifiersObjOnAct')
                svClassifiersObjOnAct(LIB.Libsvm, VS, PATH, LabelSet, ...
                    SB, paramList(i).ObjLabel, paramList(i).ActLabel, ...
                    paramList(i).trainFlag, paramList(i).savepath);
            end
        end
    end
    
end


%% This module is used to test segment classifiers and get unary potential.

if Flag.segment.potential
    
    paramList = [];
    for i=1:length(VS.vid)
        if VS.test(i) == 1
            paramList(end+1).funcName = 'svPotential';
            paramList(end).model_path = fullfile(PATH.svTSPClsf, 'Obj');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'Obj', [VS.vid{i},'.mat']);
            
            paramList(end+1).funcName = 'svPotential';
            paramList(end).model_path = fullfile(PATH.svTSPClsf, 'Act');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'Act', [VS.vid{i},'.mat']);
            
            paramList(end+1).funcName = 'svPotential';
            paramList(end).model_path = fullfile(PATH.svTSPClsf, 'BK');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'BK', [VS.vid{i},'.mat']);
            
            paramList(end+1).funcName = 'svPotentialAllSet';
            paramList(end).model_path = fullfile(PATH.svTSPClsf, 'ObjAct');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'ObjAct', [VS.vid{i},'.mat']);
            
            paramList(end+1).funcName = 'svPotentialAllSet';
            paramList(end).model_path = ...
                fullfile(PATH.svTSPClsf, 'ActOnObj');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'ActOnObj', [VS.vid{i},'.mat']);
            
            paramList(end+1).funcName = 'svPotentialAllSet';
            paramList(end).model_path = ...
                fullfile(PATH.svTSPClsf, 'ObjOnAct');
            paramList(end).svDescBase_path = fullfile(PATH.svTSPDesc, ...
                'base', [VS.vid{i}, '.mat']);
            paramList(end).svDescDt_path = fullfile(PATH.svTSPDesc, ...
                'dt', [VS.vid{i}, '.mat']);
            paramList(end).savepath = fullfile(PATH.svTSPPotn, ...
                'ObjOnAct', [VS.vid{i},'.mat']);
        end
    end
    
    fprintf('Compute Segment Unary for all videos ...\n')
    parfor i=1:length(paramList)
        if ~exist(paramList(i).savepath, 'file')
            if strcmp(paramList(i).funcName, 'svPotential')
                svPotential(LIB.Libsvm, paramList(i).model_path, ...
                    paramList(i).svDescBase_path, ...
                    paramList(i).svDescDt_path, paramList(i).savepath);
            end
            if strcmp(paramList(i).funcName, 'svPotentialAllSet')
                svPotentialAllSet(LIB.Libsvm, LabelSet, ...
                    paramList(i).model_path, ...
                    paramList(i).svDescBase_path, ...
                    paramList(i).svDescDt_path, paramList(i).savepath);
            end
        end
    end
    
end





