function labelIDX = biInfer_UGM(lib, ...
    labelMap_upper, unary_upper, Bmap_upper, Bvalue_upper, ...
    labelMap_lower, unary_lower, Bmap_lower, Bvalue_lower, ...
    link_u2l, savepath, w)


%% w is the parameter.
% w(1), w(2): object layer parameters
% w(3), w(4): action layer parameters
% w(5): ObjAct Link parameter
% w(6): object nodes parameter
% w(7): action nodes parameter

addpath(genpath(lib));

%% Node Potential
num_nodes = size(unary_upper,1);
num_states_upper = length(labelMap_upper);
num_states_lower = length(labelMap_lower);
nStates = cat(2, ones(1,num_nodes)*num_states_upper, ...
    ones(1,num_nodes)*num_states_lower);
num_states = max(nStates);

if num_states_upper < num_states
    unary_upper(num_nodes,num_states) = 0;
end
if num_states_lower < num_states
    unary_lower(num_nodes,num_states) = 0;
end 
nodePot = cat(1, unary_upper.*w(6), unary_lower.*w(7));


%% Edge Potential
% adj_upper = cat(2, zeros(num_nodes), eye(num_nodes));
% adj_lower = cat(2, eye(num_nodes), zeros(num_nodes));
adj_upper = cat(2, Bmap_upper, eye(num_nodes));
adj_lower = cat(2, eye(num_nodes), Bmap_lower);
adj = cat(1, adj_upper, adj_lower);
edgeStruct = UGM_makeEdgeStruct(adj,nStates);

edgePot = zeros(num_states,num_states,edgeStruct.nEdges);
for e = 1:edgeStruct.nEdges
    n1 = edgeStruct.edgeEnds(e,1);
    n2 = edgeStruct.edgeEnds(e,2);
    if n1<=num_nodes && n2<=num_nodes
        % Obj layer
        pot_same = eye(num_states_upper);
        pot_other = ~eye(num_states_upper).*Bvalue_upper(n1,n2);
        % tmp_mat = ones(LabelSet.objNum+1);
        tmp_mat = (pot_same+pot_other).*w(1);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat;
    elseif n1>num_nodes && n2>num_nodes
        % Act layer
        pot_same = eye(num_states_lower);
        pot_other = ~eye(num_states_lower).*Bvalue_lower(n1-num_nodes,n2-num_nodes);
        % tmp_mat = ones(LabelSet.actNum+1);
        tmp_mat = (pot_same+pot_other).*w(3);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat;
    elseif n1<=num_nodes && n2>num_nodes
        % ObjAct Link
        % tmp_mat = edgeJoint_ObjAct(BK.potential(n1), ObjAct.potential(n1,:));
        tmp_mat = link_u2l(:,:,n1);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat.*w(5);
    else
        fprintf('Impossible!\n');
    end
end

%% Inference

nodeBel = UGM_Infer_LBP(nodePot,edgePot,edgeStruct);
[~, labelIDX_upper] = max(nodeBel(1:num_nodes,:),[],2);
[~, labelIDX_lower] = max(nodeBel(num_nodes+1:end,:),[],2);

% nodeDecode = UGM_Decode_ICM(nodePot,edgePot,edgeStruct);
% nodeDecode = double(nodeDecode);
% labelIDX_upper = nodeDecode(1:num_nodes);
% labelIDX_lower = nodeDecode(num_nodes+1:end);

labelIDX_upper = labelIDX_upper-1;
labelIDX_lower = labelIDX_lower-1;
labelIDX = labelIDX_upper*10+labelIDX_lower;
labelIDX(labelIDX<10) = 0;
labelIDX(labelIDX==0) = 1;

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'labelIDX', '-v7.3');

% function Pot_mat = edgeJoint_ObjAct(BK_pot, ObjAct_pot)
% Pot_mat = reshape(ObjAct_pot, LabelSet.actNum, LabelSet.objNum);
% Pot_mat = Pot_mat';
% Pot_mat = cat(1, zeros(1,LabelSet.actNum), Pot_mat);
% Pot_mat = cat(2, [BK_pot; zeros(LabelSet.objNum,1)], Pot_mat);

    
