function labelIDX = triInfer_UGM(lib, ...
    labelMap_middle, unary_middle, Bmap_middle, Bvalue_middle, ...
    labelMap_upper, unary_upper, Bmap_upper, Bvalue_upper, ...
    labelMap_lower, unary_lower, Bmap_lower, Bvalue_lower, ...
    link_m2u, link_m2l, savepath, w)


%% w is the parameter.
% w(1), w(2): ObjAct layer parameters
% w(3), w(4): object layer parameters
% w(5), w(6): action layer parameters
% w(7): ObjAct -- Obj Link parameter
% w(8): ObjAct -- Act Link parameter
% w(9): ObjAct nodes parameter
% w(10): Object nodes parameter
% w(11): Action nodes parameter

addpath(genpath(lib));

%% Node Potential
num_nodes = size(unary_middle,1);
num_states_middle = length(labelMap_middle);
num_states_upper = length(labelMap_upper);
num_states_lower = length(labelMap_lower);
nStates = cat(2, ones(1,num_nodes)*num_states_middle, ...
    ones(1,num_nodes)*num_states_upper, ...
    ones(1,num_nodes)*num_states_lower);
num_states = max(nStates);

if num_states_middle < num_states
    unary_middle(num_nodes,num_states) = 0;
end
if num_states_upper < num_states
    unary_upper(num_nodes,num_states) = 0;
end
if num_states_lower < num_states
    unary_lower(num_nodes,num_states) = 0;
end 
nodePot = cat(1, unary_middle.*w(9), unary_upper.*w(10), unary_lower.*w(11));


%% Edge Potential

% adj_middle = cat(2, zeros(num_nodes), eye(num_nodes), eye(num_nodes));
% adj_upper = cat(2, eye(num_nodes), zeros(num_nodes), zeros(num_nodes));
% adj_lower = cat(2, eye(num_nodes), zeros(num_nodes), zeros(num_nodes));
adj_middle = cat(2, Bmap_middle, eye(num_nodes), eye(num_nodes));
adj_upper = cat(2, eye(num_nodes), Bmap_upper, zeros(num_nodes));
adj_lower = cat(2, eye(num_nodes), zeros(num_nodes), Bmap_lower);
adj = cat(1, adj_middle, adj_upper, adj_lower);
edgeStruct = UGM_makeEdgeStruct(adj,nStates);

edgePot = zeros(num_states,num_states,edgeStruct.nEdges);
for e = 1:edgeStruct.nEdges
    n1 = edgeStruct.edgeEnds(e,1);
	n2 = edgeStruct.edgeEnds(e,2);
    if n1<=num_nodes && n2<=num_nodes
        % ObjAct layer
        pot_same = eye(num_states_middle);
        pot_other = ~eye(num_states_middle).*Bvalue_middle(n1,n2);
        % tmp_mat = ones(num_states_upper);
        tmp_mat = (pot_same+pot_other).*w(1);
        edgePot(:,:,e) = tmp_mat;
    elseif n1>num_nodes && n1<=2*num_nodes && n2>num_nodes && n2 <=2*num_nodes
        % Obj layer
        pot_same = eye(num_states_upper);
        pot_other = ~eye(num_states_upper).*Bvalue_upper(n1-num_nodes,n2-num_nodes);
        % tmp_mat = ones(num_states_middle);
        tmp_mat = (pot_same+pot_other).*w(3);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat;
    elseif n1>2*num_nodes && n2>2*num_nodes
        % Act layer
        pot_same = eye(num_states_lower);
        pot_other = ~eye(num_states_lower).*Bvalue_lower(n1-2*num_nodes,n2-2*num_nodes);
        % tmp_mat = ones(num_states_lower);
        tmp_mat = (pot_same+pot_other).*w(5);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat;
    elseif n1<=num_nodes && n2>num_nodes && n2<=2*num_nodes
        % ObjAct -- Obj Links
        tmp_mat = link_m2u(:,:,n1);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) = tmp_mat.*w(7);
    elseif n1<=num_nodes && n2>2*num_nodes
        % ObjAct -- Act Links
        tmp_mat = link_m2l(:,:,n1);
        tmp_mat(num_states,num_states) = 0;
        edgePot(:,:,e) =tmp_mat.*w(8);
    elseif n1>num_nodes && n1<=2*num_nodes && n2<=num_nodes
        fprintf('No need to set ... \n');
    elseif n1>2*num_nodes && n2<=num_nodes
        fprintf('No need to set ... \n');
    else
        fprintf('Impossible!\n');
    end
end

%% Inference

nodeBel = UGM_Infer_LBP(nodePot,edgePot,edgeStruct);
[~, labelIDX] = max(nodeBel(1:num_nodes,:),[],2);

labelIDX = labelMap_middle(labelIDX);

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'labelIDX', '-v7.3');



