function  [gtMap, gtList, gtPos] = loadGroundTruth(gtColor, gtFold, scale, savepath)
% This function is used to read CamVid ground-truth.
% gtColor: Nx3 matrix, rows are 1:N labels, colomns are RGB.
% gtFold: ground-truth folder of a video which contains a set of 
%   png images, with each RGB color denoting a label.
% scale: the scale of ground-truth for evaluation. For example, 0.5 means 
%   half of the original resolution.
% gtMap: uint16 3D volume with 0 denoting void pixel and >0 denoting
%   objects in the order of gtColor.
% gtList: array of ground-truth segment index in gtMap.
% gtPos: the frame id of ground-truth in a video


% To initialize gtMap
gtImList = dir(fullfile(gtFold, '*.png'));
gtIm = imread(fullfile(gtFold, gtImList(1).name));
[I_h, I_w, ~] = size(gtIm);
gtMap = zeros(I_h, I_w, length(gtImList));
gtPos = zeros(length(gtImList),1);

for i=1:length(gtImList)
    gtIm = imread(fullfile(gtFold, gtImList(i).name));
    gtIm = reshape(gtIm,[],3);
    [~, Locb] = ismember(gtIm, gtColor, 'rows');
    gtIm = Locb; % Note void is 0
    gtIm = reshape(gtIm,I_h,I_w);
    gtMap(:,:,i) = gtIm;
    gtPos(i) = str2double(gtImList(i).name(1:end-4));
end

if nargin > 2
    if scale ~= 1
        gtMap_org = gtMap;
        gtMap = [];
        for i=1:size(gtMap_org,3)
            gtMap = cat(3, gtMap, imresize(gtMap_org(:,:,i), scale, 'nearest'));
        end
    end
end

% Get gtList
gtList = unique(gtMap);
gtList = gtList(gtList>0);

if nargin > 3
    save(savepath, 'gtMap', 'gtList', 'gtPos', '-v7.3');
end
