function [VS, PATH, Param, LabelSet]  = setupDatasetA2D(codeRoot)

LCDATA = fullfile(codeRoot,'..','Data','A2D');
LCRELT = fullfile(codeRoot,'..','Results','A2D');


%% VideoSet
VS.name = 'A2D';

VS.csvPath = fullfile(codeRoot,'videoset.csv');

VS.orgVideoPath = fullfile(LCDATA,'videos');
VS.orgClipPath = fullfile(LCDATA,'clips');
VS.orgPngPath = fullfile(LCDATA,'pngs');
VS.orgJpgPath = fullfile(LCDATA,'jpgs');
VS.orgThumbPath = fullfile(LCDATA,'thumb');

VS.clipPath = fullfile(LCDATA,'clips320Hnew');
VS.pngPath = fullfile(LCDATA,'pngs320H');
VS.ppmPath = fullfile(LCDATA,'ppms320H');
VS.jpgPath = fullfile(LCDATA,'jpgs320H');

VS.gtXmlPath = fullfile(LCDATA,'Annotations','xml');
VS.gtMatPath = fullfile(LCDATA,'Annotations','mat');
VS.gtPixIdPath = fullfile(LCDATA,'Annotations','id');
VS.gtPixColPath = fullfile(LCDATA,'Annotations','col');

VS = readInfo(VS);


%% Features

PATH.ftMr = fullfile(LCRELT,'Features','mr'); % Quantized MR
PATH.ftMrBook = fullfile(LCRELT,'Features','mr.mat'); % codebook 100

PATH.ftSift = fullfile(LCRELT,'Features','sift'); % Quantized SIFT
PATH.ftSiftBook = fullfile(LCRELT,'Features','sift.mat'); % codebook 100

PATH.ftFlowRaw = fullfile(LCRELT,'Features','flow_raw'); % RAW
PATH.ftFlow = fullfile(LCRELT,'Features','flow'); % Quantized Flow
PATH.ftFlowBook = fullfile(LCRELT,'Features','flow.mat');

PATH.ftDtRaw = fullfile(LCRELT,'Features','dt_raw'); % RAW
PATH.ftDt = fullfile(LCRELT,'Features','dt'); % Quantized DT
PATH.ftDtBook = fullfile(LCRELT,'Features','dt.mat'); % codebook 100 * 5


%% Supervoxel - TSP 400
PATH.svTSPMat = fullfile(LCRELT,'SegmentsTSP400','svMap');
PATH.svTSPDesc = fullfile(LCRELT,'SegmentsTSP400','svDesc');
PATH.svTSPAffy = fullfile(LCRELT,'SegmentsTSP400','svAffy');
PATH.svTSPPotn = fullfile(LCRELT,'SegmentsTSP400','svPotn');
PATH.svTSPClsf = fullfile(LCRELT,'SegmentsTSP400','svClsf');
PATH.svTSPGTmat = fullfile(LCRELT,'SegmentsTSP400','svGT.mat');


%% Models
PATH.modFlat = fullfile(LCRELT,'Models','segLabels_Flat');
PATH.modBilayer = fullfile(LCRELT,'Models','segLabels_Bilayer');
PATH.modTrilayer = fullfile(LCRELT,'Models','segLabels_Trilayer');
PATH.modCondition = fullfile(LCRELT,'Models','segLabels_Holistic');


%% Visualization
PATH.vizFlat = fullfile(LCRELT,'Visual','segLabels_Flat');
PATH.vizBilayer = fullfile(LCRELT,'Visual','segLabels_Bilayer');
PATH.vizTrilayer = fullfile(LCRELT,'Visual','segLabels_Trilayer');
PATH.vizCondition = fullfile(LCRELT,'Visual','segLabels_Holistic');


%% Parameters

Param.feature.rgb = 8;
Param.feature.hsv = 8;

Param.feature.mr.ndata_max = 4000000;
Param.feature.mr.fpv = 10;
Param.feature.mr.dsize = 100;

Param.feature.sift.ndata_max = 4000000;
Param.feature.sift.fpv = 10;
Param.feature.sift.dsize = 100;

Param.feature.flow.sampleNum = 100000;
Param.feature.flow.x_num = 16;
Param.feature.flow.y_num = 16;
Param.feature.flow.mag_num = 16;

Param.feature.dt.ndata_max = 100000;
Param.feature.dt.dsize = 100;

Param.segment.tsp.k = 400;

% alpha: controls weight between unary and binary, the larger the stronger binary
% theta: controls weight in binary agreement, the larger the more difference

Param.model.flat.obj.alpha = 0.5;
Param.model.flat.obj.theta = 0.2;
Param.model.flat.act.alpha = 0.4;
Param.model.flat.act.theta = 0.4;
Param.model.flat.objact.alpha = 0.5;
Param.model.flat.objact.theta = 0.3;

Param.model.bilayer.obj.alpha = 0.5;
Param.model.bilayer.obj.theta = 0.2;
Param.model.bilayer.act.alpha = 0.4;
Param.model.bilayer.act.theta = 0.4;
Param.model.bilayer.obj.weight = 1;
Param.model.bilayer.objact.weight = 1;
Param.model.bilayer.act.weight = 1;

Param.model.trilayer.objact.alpha = 0.5;
Param.model.trilayer.objact.theta = 0.3;
Param.model.trilayer.obj.alpha = 0.5;
Param.model.trilayer.obj.theta = 0.2;
Param.model.trilayer.act.alpha = 0.4;
Param.model.trilayer.act.theta = 0.4;
Param.model.trilayer.objact2obj.weight = 1;
Param.model.trilayer.objact2act.weight = 1;
Param.model.trilayer.objact.weight = 1;
Param.model.trilayer.obj.weight = 1;
Param.model.trilayer.act.weight = 1;


%% LabelSet

% Background
LabelSet.bk = 1;

% Object Set
% 1-adult, 2-baby, 3-ball, 4-bird, 5-car, 6-cat, 7-dog
LabelSet.obj = [1;2;3;4;5;6;7;];
LabelSet.objNum = 7;
LabelSet.objName = {'adult';'baby';'ball';'bird';'car';'cat';'dog'};

% Action Set
% 1-climbing, 2-crawling, 3-eating, 4-flying, 5-jumping, 
% 6-rolling, 7-running, 8-walking, 9-none/mixed
LabelSet.act = [1;2;3;4;5;6;7;8;9;]; 
LabelSet.actNum = 9;
LabelSet.actName = {'climbing';'crawling';'eating';'flying';'jumping'; ...
    'rolling';'running';'walking';'none'};

% Action Recognition Set
LabelSet.ar = [
    11;12;13;   15;16;17;18;
    21;22;         26;   28;
             34;35;36;
    41;   43;44;45;46;   48;
             54;55;56;57;
    61;   63;   65;66;67;68;
       72;73;   75;76;77;78;];
LabelSet.arNum = 36;

% Joint Set
LabelSet.joint = [
    11;12;13;   15;16;17;18;19;
    21;22;         26;   28;29;
             34;35;36;      39;
    41;   43;44;45;46;   48;49;
             54;55;56;57;   59;
    61;   63;   65;66;67;68;69;
       72;73;   75;76;77;78;79;];
LabelSet.jointNum = 43;    

% All Label Set
LabelSet.allSet = [
    11;12;13;14;15;16;17;18;19;
    21;22;23;24;25;26;27;28;29;
    31;32;33;34;35;36;37;38;39;
    41;42;43;44;45;46;47;48;49;
    51;52;53;54;55;56;57;58;59;
    61;62;63;64;65;66;67;68;69;
    71;72;73;74;75;76;77;78;79;];
LabelSet.allSetNum = 63;
LabelSet.allSetValid = ismember(LabelSet.allSet, LabelSet.joint);
LabelSet.allSetOut = [
    [1,0]; [2,0]; [3,0]; [4,0]; [5,0]; [6,0]; [7,0]; [8,0]; [9,0]; ...
    [10,19]; [20,29]; [30,39]; [40,49]; [50,59]; [60,69]; [70,79];
    ];


% Color Space for Labels
[LabelSet.C, LabelSet.H] = vsColMap();


%% Read Video Sheet
function VS = readInfo(VS)
% This script is used to manage the csv file.
% 
% csv file structure:
% vid, label, timestart, timeend, height, width, frames, anno, test
% vid: 11-digit YouTube video id.
% label: single video-level label (e.g. 17 (adult-running)).
% timestart: start time to cut for a clip out of a video.
% timeend: end time for the clip.
% height: resized height.
% width: resized width.
% frames: number of frames of the cut clip.
% anno: number of VALID annotation frames.
% test: 0 - train; 1 - test.
% 

fid = fopen(VS.csvPath, 'r');
C = textscan(fid, '%s%d%s%s%d%d%d%d%d', 'delimiter', ',');
fclose(fid);

VS.vid = C{1};
VS.label = C{2};
VS.timeStart = C{3};
VS.timeEnd = C{4};
VS.height = C{5};
VS.width = C{6};
VS.frames = C{7};
VS.anno = C{8};
VS.test = C{9};


%% Construct Color Space
function [C, H] = vsColMap()

H1 = [0,1,0.2;
    0,1,0.4;
    0,1,0.6;
    0,1,0.8;
    0,1,1;
    0,0.8,1;
    0,0.6,1;
    0,0.4,1;
    0,0.2,1];
H2=H1; H3=H1; H4=H1; H5=H1; H6=H1; H7=H1;
H2(H2==0)=0.15;
H3(H3==0)=0.3;
H4(H4==0)=0.45;
H5(H5==0)=0.6;
H6(H6==0)=0.75;
H7(H7==0)=0.9;

C1 = hsv2rgb(H1);
C2 = hsv2rgb(H2);
C3 = hsv2rgb(H3);
C4 = hsv2rgb(H4);
C5 = hsv2rgb(H5);
C6 = hsv2rgb(H6);
C7 = hsv2rgb(H7);

H = cat(3,H1,H2,H3,H4,H5,H6,H7);
clear H1 H2 H3 H4 H5 H6 H7

C = cat(3,C1,C2,C3,C4,C5,C6,C7);
clear C1 C2 C3 C4 C5 C6 C7

C = uint8(C*255 + 0.5);
% for i=1:size(C,3)
%     figure;
%     tmp_c = C(:,:,i);
%     image(reshape(tmp_c, [1 size(tmp_c)]));
% end
