function centers = ftDtBook(lib, fpath_list, savepath, param)

ndata_max = param.ndata_max;
dsize = param.dsize;

vnum = length(fpath_list);
pnum = floor(ndata_max/vnum);
fprintf('Sample %d videos with %d points each. \n', vnum, pnum);

fullresp = [];
bad_videos = [];
bad_cnt = 0;
for i=1:vnum
    filename = fpath_list{i};
    try
        data2add = load(filename);
    catch error
        bad_cnt = bad_cnt + 1;
        bad_videos{bad_cnt} = VS.vid{i};
    end
    if ~isempty(data2add.raw)
        if (size(data2add.raw,1) > pnum)
            p = randperm(size(data2add.raw,1));
            data2add.raw = data2add.raw(p(1:pnum),:);
        end
        fullresp = [fullresp; data2add.raw];
    end
    if mod(i,10) == 0
        fprintf('%d videos processed...\n', i);
    end
end

% Make fullresp a 426-D feature vector matrix
fullresp = double(fullresp);
fullresp = fullresp(:,11:end-30);

type_list = {'traj';'hog';'hof';'mbhx';'mbhy'};
centers_list = [];
parfor i=1:length(type_list)
    type = type_list{i};
    switch type
        case 'traj' % (default 30 dimension)
            featSamples = fullresp(:,1:30);
        case 'hog' % (default 96 dimension)
            featSamples = fullresp(:,31:126);
        case 'hof' % (default 108 dimension)
            featSamples = fullresp(:,127:234);
        case 'mbhx' % (default 96 dimension)
            featSamples = fullresp(:,235:330);
        case 'mbhy' % (default 96 dimension)
            featSamples = fullresp(:,331:426);
    end
    
    fprintf('Running K-means ...\n');
    opts = statset('MaxIter',200,'Display','iter');
    [~, centers_list{i}] = kmeans(featSamples, dsize, 'Options', opts);
end

centers.traj = centers_list{1};
centers.hog = centers_list{2};
centers.hof = centers_list{3};
centers.mbhx = centers_list{4};
centers.mbhy = centers_list{5};

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'centers', '-v7.3');















