function flowHist = ftFlow(lib, center_mat, fpath, savepath, param)
% This function is used to quantize optical flow raw feature.
% Quantization: 8 bin x, 8 bin y, 16 magnitude

% load centers.x centers.y centers.mag
load(center_mat);

mlist = dir(fullfile(fpath,'*.mat'));
mnum = length(mlist);
load(fullfile(fpath, mlist(1).name));
[I_h, I_w] = size(flow.fvx);
flowHist.x = zeros(I_h,I_w,mnum);
flowHist.y = zeros(I_h,I_w,mnum);
flowHist.mag = zeros(I_h,I_w,mnum);

for i=1:length(mlist)
    % fprintf('Processing mat: %d ...\n', i);
    % Load flow.fvx flow.fvy flow.bvx flow.bvy
    load(fullfile(fpath, mlist(i).name));
    
    IDX = knnsearch(centers.x, flow.fvx(:));
    flowHist.x(:,:,i) = reshape(IDX, [I_h, I_w]);
    
    IDX = knnsearch(centers.y, flow.fvy(:));
    flowHist.y(:,:,i) = reshape(IDX, [I_h, I_w]);
    
    flow_mag = sqrt(flow.fvx.^2+flow.fvy.^2);
    IDX = knnsearch(centers.mag, flow_mag(:));
    flowHist.mag(:,:,i) = reshape(IDX, [I_h, I_w]);
end

flowHist.x = uint8(flowHist.x);
flowHist.y = uint8(flowHist.y);
flowHist.mag = uint8(flowHist.mag);

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'flowHist', '-v7.3');