function centers = ftMrBook(lib, fpath_list, savepath, param)

addpath(genpath(lib));

ndata_max = param.ndata_max; % total number of sampled points
fpv = param.fpv; % number of sampled frames per video
dsize = param.dsize; % codebook size

f_total = length(fpath_list)*fpv;
fullresp = [];
for i=1:length(fpath_list)
    flist = dir(fullfile(fpath_list{i},'*.png'));
    fnum = length(flist);
    pframe = randperm(fnum, min(fnum,fpv));
    for k=1:length(pframe)
        im = imread(fullfile(fpath_list{i}, flist(pframe(k)).name));
        [foo, data2add] = FullMR8(im);
        if(size(data2add,2)>ndata_max/f_total)
            p = randperm(size(data2add,2));
            data2add = data2add(:,p(1:floor(ndata_max/f_total)));
        end
        fullresp = [fullresp data2add];
    end
    if mod(i,10) == 0
        fprintf('%d videos processed...\n', i);
    end
end
fullresp = fullresp';

%% run kmeans
fprintf('\nRunning k-means\n');
opts = statset('MaxIter',100,'Display','iter');
[foo, centers] = kmeans(fullresp, dsize,'Options',opts);

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'centers', '-v7.3');