function [svMap, svNum] = readSegmentation(IN_path, fmt)
% [svMap, svNum] = readSegmentation(IN_path, fmt)
% IN_path: path to output segmentation frames encoded in RGB color, 
%   each segment has an unique color
% fmt: frame format png or ppm
% svMap: supervoxel index map I_h x I_w x frame_num
% svNum: number of supervoxels

imList = dir(fullfile(IN_path, ['*.',fmt]));
if isempty(imList)
    svMap = []; svNum = [];
    fprintf('No frame (%s) in %s \n', fmt, IN_path);
    return
end

[I_h, I_w, ~] = size(imread(fullfile(IN_path, imList(1).name)));
svMap = zeros(I_h, I_w, length(imList));
for i=1:length(imList)
    sv_im = imread(fullfile(IN_path, imList(i).name));
    svMap(:,:,i) = im_RGB2ID(sv_im);
end
svMap = cvlbmap(svMap);
svMap = uint32(svMap);
svNum = size(unique(svMap),1);


function im_ID = im_RGB2ID(im_RGB)
% im_ID = im_RGB2ID(im_RGB)
% The cluster IDs are coded on three bytes: red-green-blue. For exmaple if 
% a pixel value is (255, 0, 99), then its index value = 255 * 256^2 + 
% 0 * 256 + 99
im_RGB = double(im_RGB);
im_ID = im_RGB(:,:,1) * 256 * 256 +...
    im_RGB(:,:,2) * 256 +...
    im_RGB(:,:,3);


function Y = cvlbmap(X)
% convert labels in X to 1:N
X = double(X);
X = X + 1;
u = unique(X);
SparseIndex = sparse(u,1,1:length(u),max(u),1);
Y = full(SparseIndex(X(:)));
Y = reshape(Y,size(X));