function model = svClassifiersBK(lib, VS, PATH, LabelSet, SB, trainFlag, savepath)
% This function is used to train and test segment level svm classifiers.
% labelID = Obj 1-7 / Act 1-9 / ObjAct 11-79
% classType = 'Obj'/'Act'/'ObjAct'
% featType = 'base'/'dt'/'all'
% trainFlag: 1 --> training, 0 --> testing svm performance

SampleNum_pos = 10000;
SampleNum_neg = 10000;

if ischar(trainFlag)
    trainFlag = str2double(trainFlag);
end

%% positive samples
fprintf('Sampling positive instances ... \n');
posPool = getPool(VS, SB, 'BK', 1, trainFlag);
if size(posPool,1) > SampleNum_pos
    p = randperm(size(posPool,1));
    posSamples = posPool(p(1:SampleNum_pos),:);
else
    posSamples = posPool;
end

%% negative samples
fprintf('Sampling Other negative instances ... \n');
SampleNum = floor(SampleNum_neg/length(LabelSet.joint));
negSamples = [];
for i=1:length(LabelSet.joint)
    OtherPool = getPool(VS, SB, 'ObjAct', LabelSet.joint(i), trainFlag);
    if size(OtherPool,1) > SampleNum
        p = randperm(size(OtherPool,1));
        negSamples = [negSamples; OtherPool(p(1:SampleNum),:)];
    else
        negSamples = [negSamples; OtherPool];
    end
end

%% Extract Feature Vectors
fprintf('Extracting Feature Vectors ... \n');
[posFeat, negFeat] = getFeatVect(VS, PATH, posSamples, negSamples);

%% Training
fprintf('Sampled positive instances: %d\n', size(posFeat,1));
fprintf('Sampled negative instances: %d\n', size(negFeat,1));
if trainFlag
    fprintf('Training Model ... \n');
    model = svmtraining(lib, 1, 'BK', posFeat, negFeat);
    test = [];
    savefold = fileparts(savepath);
    if ~exist(savefold, 'dir')
        mkdir(savefold);
    end
    save(savepath, 'model', '-v7.3');
else
    fprintf('Testing Model Performance ... \n');
    load(savepath);
    test = svmtesting(lib, model, posFeat, negFeat);
end






        
        
