function [model, test] = svClassifiersObjOnAct(lib, VS, PATH, LabelSet, SB, ObjLabel, ActLabel, trainFlag, savepath)
% This function is used to train conditional classifiers: Obj|Act.

SampleNum_pos = 5000;
SampleNum_neg = 5000;

if ischar(ObjLabel)
    ObjLabel = str2double(ObjLabel);
end
if ischar(ActLabel)
    ActLabel = str2double(ActLabel);
end
if ischar(trainFlag)
    trainFlag = str2double(trainFlag);
end

%% label Set
% Find all objects that can perform this action
labelScope = LabelSet.joint(mod(LabelSet.joint,10)==ActLabel);

% Check if the desired Act|Obj is valid
labelID = ObjLabel*10 + ActLabel;
if isempty(find(labelScope==labelID,1))
    model = []; test = [];
    return
end
labelOthers = labelScope(labelScope~=labelID);

%% Positive Samples
fprintf('Sampling positive instances ... \n');
posPool = getPool(VS, SB, 'ObjAct', labelID, trainFlag);
if size(posPool,1) > SampleNum_pos
    p = randperm(size(posPool,1));
    posSamples = posPool(p(1:SampleNum_pos),:);
else
    posSamples = posPool;
end
if isempty(posSamples)
    fprintf('No positive training instances for ObjOnAct %d\n', labelID);
    model = []; test = [];
    return
end

%% Negative Samples
fprintf('Sampling negative instances ... \n');
SampleNum = floor(SampleNum_neg/length(labelOthers));
negSamples = [];
for i=1:length(labelOthers)
    OtherPool = getPool(VS, SB, 'ObjAct', labelOthers(i), trainFlag);
    if size(OtherPool,1) > SampleNum
        p = randperm(size(OtherPool,1));
        negSamples = [negSamples; OtherPool(p(1:SampleNum),:)];
    else
        negSamples = [negSamples; OtherPool];
    end
end

%% Extract Feature Vectors
fprintf('Extracting Feature Vectors ... \n');
[posFeat, negFeat] = getFeatVect(VS, PATH, posSamples, negSamples);

%% Training
fprintf('Sampled positive instances: %d\n', size(posFeat,1));
fprintf('Sampled negative instances: %d\n', size(negFeat,1));
if trainFlag
    fprintf('Training Model ... \n');
    model = svmtraining(lib, labelID, 'ObjOnAct', posFeat, negFeat);
    test = [];
    savefold = fileparts(savepath);
    if ~exist(savefold, 'dir')
        mkdir(savefold);
    end
    save(savepath, 'model', '-v7.3');
else
    fprintf('Testing Model Performance ... \n');
    load(savepath);
    test = svmtesting(lib, model, posFeat, negFeat);
end



