function fv = svDescBase(frame_path, frame_fmt, svMap_path, ...
    ftMr_path, ftSift_path, ftFlow_path, savepath, param)

%% Load Video Segmentation
load(svMap_path);
segmentNum = length(unique(svMap));
segments(segmentNum).frame = [];
segments(segmentNum).size = [];
segments(segmentNum).size_Dilate = [];
segments(segmentNum).rgbHist_Dilate = [];
segments(segmentNum).hsvHist_Dilate = [];
segments(segmentNum).mrHist_Dilate = [];
segments(segmentNum).siftHist_Dilate = [];
segments(segmentNum).flowHist_x_Dilate = [];
segments(segmentNum).flowHist_y_Dilate = [];
segments(segmentNum).flowHist_mag_Dilate = [];

%% Load Various Quantized Features
fprintf('load video folder. \n');
frameList = dir(fullfile(frame_path, ['*.',frame_fmt]));
frameNum = length(frameList);
fprintf('Load quantized single 3D mr feature video. \n');
mr = load(ftMr_path);
mr.textons_video = double(mr.textons_video);
fprintf('Load quantized single 3D sift feature video. \n');
sift = load(ftSift_path);
sift.textons_video = double(sift.textons_video);
fprintf('Load uint8 frameNum-1 flowHist.x flowHist.y flowHist.mag. \n');
load(ftFlow_path);
flowHist.x(:,:,end+1) = flowHist.x(:,:,end); flowHist.x = double(flowHist.x);
flowHist.y(:,:,end+1) = flowHist.y(:,:,end); flowHist.y = double(flowHist.y);
flowHist.mag(:,:,end+1) = flowHist.mag(:,:,end); flowHist.mag = double(flowHist.mag);

centerNum.rgb = param.rgb;
centerNum.hsv = param.hsv;
centerNum.mr = param.mr.dsize;
centerNum.sift = param.sift.dsize;
centerNum.flow_x = param.flow.x_num;
centerNum.flow_y = param.flow.y_num;
centerNum.flow_mag = param.flow.mag_num;

base.rgbHist_Dilate = zeros(segmentNum, centerNum.rgb*3);
base.hsvHist_Dilate = zeros(segmentNum, centerNum.hsv*3);
base.mrHist_Dilate = zeros(segmentNum, centerNum.mr);
base.siftHist_Dilate = zeros(segmentNum, centerNum.sift);
base.flowHist_x_Dilate = zeros(segmentNum, centerNum.flow_x);
base.flowHist_y_Dilate = zeros(segmentNum, centerNum.flow_y);
base.flowHist_mag_Dilate = zeros(segmentNum, centerNum.flow_mag);

%% Go over each frame to get features
for i=1:frameNum
    fprintf('processing frame: %d\n', i);
    segmentList = unique(svMap(:,:,i));
    rgb_im = imread(fullfile(frame_path, frameList(i).name));
    hsv_im = rgb2hsv(rgb_im);
    mr_im = mr.textons_video(:,:,i);
    sift_im = sift.textons_video(:,:,i);
    flowHist_x_im = flowHist.x(:,:,i);
    flowHist_y_im = flowHist.y(:,:,i);
    flowHist_mag_im = flowHist.mag(:,:,i);
    
    for k=1:length(segmentList)
        id = segmentList(k);        
        mask = (svMap(:,:,i)==id);
        [maskDilate, bb] = getBorders(mask);
        
        maskCrop = mask(bb(1):bb(2),bb(3):bb(4));
        maskDilateCrop = maskDilate(bb(1):bb(2),bb(3):bb(4));
        rgbCrop = rgb_im(bb(1):bb(2),bb(3):bb(4),:);
        hsvCrop = hsv_im(bb(1):bb(2),bb(3):bb(4),:);
        mrCrop = mr_im(bb(1):bb(2),bb(3):bb(4));
        siftCrop = sift_im(bb(1):bb(2),bb(3):bb(4));
        flowHistCrop_x = flowHist_x_im(bb(1):bb(2),bb(3):bb(4));
        flowHistCrop_y = flowHist_y_im(bb(1):bb(2),bb(3):bb(4));
        flowHistCrop_mag = flowHist_mag_im(bb(1):bb(2),bb(3):bb(4));
        
        segments(id).frame = [segments(id).frame; i];
        segments(id).size = [segments(id).size; sum(maskCrop(:))];
        segments(id).size_Dilate = [segments(id).size_Dilate; sum(maskDilateCrop(:))];
        segments(id).rgbHist_Dilate = [segments(id).rgbHist_Dilate; getColorHist(rgbCrop, maskDilateCrop, centerNum.rgb, 'rgb')];
        segments(id).hsvHist_Dilate = [segments(id).hsvHist_Dilate; getColorHist(hsvCrop, maskDilateCrop, centerNum.hsv, 'hsv')];
        segments(id).mrHist_Dilate = [segments(id).mrHist_Dilate; getMaskHist(mrCrop, maskDilateCrop, centerNum.mr)];
        segments(id).siftHist_Dilate = [segments(id).siftHist_Dilate; getMaskHist(siftCrop, maskDilateCrop, centerNum.sift)];
        segments(id).flowHist_x_Dilate = [segments(id).flowHist_x_Dilate; getMaskHist(flowHistCrop_x, maskDilateCrop, centerNum.flow_x)];
        segments(id).flowHist_y_Dilate = [segments(id).flowHist_y_Dilate; getMaskHist(flowHistCrop_y, maskDilateCrop, centerNum.flow_y)];
        segments(id).flowHist_mag_Dilate = [segments(id).flowHist_mag_Dilate; getMaskHist(flowHistCrop_mag, maskDilateCrop, centerNum.flow_mag)];
    end
end

%% Organize Segment Feature Vectors
for i=1:segmentNum
    base.rgbHist_Dilate(i,:) = sum(segments(i).rgbHist_Dilate,1)./sum(segments(i).size_Dilate);
    base.hsvHist_Dilate(i,:) = sum(segments(i).hsvHist_Dilate,1)./sum(segments(i).size_Dilate);
    base.mrHist_Dilate(i,:) = sum(segments(i).mrHist_Dilate,1)./sum(segments(i).size_Dilate);
    base.siftHist_Dilate(i,:) = sum(segments(i).siftHist_Dilate,1)./sum(segments(i).size_Dilate);
    base.flowHist_x_Dilate(i,:) = sum(segments(i).flowHist_x_Dilate,1)./sum(segments(i).size_Dilate);
    base.flowHist_y_Dilate(i,:) = sum(segments(i).flowHist_y_Dilate,1)./sum(segments(i).size_Dilate);
    base.flowHist_mag_Dilate(i,:) = sum(segments(i).flowHist_mag_Dilate,1)./sum(segments(i).size_Dilate);
end

fv = cat(2, base.rgbHist_Dilate, base.hsvHist_Dilate, base.mrHist_Dilate, ...
    base.siftHist_Dilate, base.flowHist_x_Dilate, ...
    base.flowHist_y_Dilate, base.flowHist_mag_Dilate);

savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'fv', '-v7.3');



function [maskDilate, bb] = getBorders(mask)

strEl = strel('square', 10);
maskDilate = imdilate(mask,strEl,'same');
[y,x] = find(maskDilate);
top = min(y);
bottom = max(y);
left = min(x);
right = max(x);
bb = [top bottom left right];


function desc = getColorHist(colorCrop, maskCrop, numBins, colorType)

switch colorType
    case 'rgb'
        binSize = 256/numBins;
        binCenters = (binSize-1)/2:binSize:255;
    case 'hsv'
        binSize = 1/numBins;
        binCenters = binSize/2:binSize:1;
    otherwise
        fprintf('Unknown color type! \n');
end

desc = [];
for c = 1:3
    r = colorCrop(:,:,c);
    desc = [desc hist(r(maskCrop),binCenters)];
end


function desc = getMaskHist(histCrop, maskCrop, DictionarySize)

maskCrop = maskCrop(:);
histCrop = histCrop(:);
histCrop(~maskCrop) = [];

if (~isempty(histCrop))
    desc =  hist(histCrop(:),1:DictionarySize);
else
    desc = zeros(1, DictionarySize);
end

