function fv = svDescDT(svMap_path, ftDt_path, savepath, param)

%% Load Video Segmentation
load(svMap_path);
segmentNum = length(unique(svMap));
[I_h, I_w, frame_num] = size(svMap);

segments(segmentNum).IDX = [];
segments(segmentNum).traj = [];
segments(segmentNum).hog = [];
segments(segmentNum).hof = [];
segments(segmentNum).mbhx = [];
segments(segmentNum).mbhy = [];

%% Load DT Features
fprintf('Load quantized uint16 DT featHist. \n');
load(ftDt_path);
featHist = double(featHist);
centerNum.traj = param.dsize;
centerNum.hog = param.dsize;
centerNum.hof = param.dsize;
centerNum.mbhx = param.dsize;
centerNum.mbhy = param.dsize;

dt.traj = zeros(segmentNum,centerNum.traj);
dt.hog = zeros(segmentNum,centerNum.hog);
dt.hof = zeros(segmentNum,centerNum.hof);
dt.mbhx = zeros(segmentNum,centerNum.mbhx);
dt.mbhy = zeros(segmentNum,centerNum.mbhy);

%% Assign each DT to the most occupied Segment
% for i=1:size(featHist,1)
%     % x is the column, y is the row
%     x = featHist(i,2:2:31); x(x>I_w)=I_w; x(x<1)=1;
%     y = featHist(i,3:2:31); y(y>I_h)=I_h; y(y<1)=1;
%     f = featHist(i,1)-14:featHist(i,1); f(f>frame_num)=frame_num; f(f<1)=1;
%     posVect = y+(x-1).*I_h+(f-1).*(I_h*I_w);
%     id = mode(svMap(posVect));
%     dt.IDX{id} = [dt.IDX{id}, i];
%     dt.traj{id} = [dt.traj{id}, featHist(i,32)];
%     dt.hog{id} = [dt.hog{id}, featHist(i,33)];
%     dt.hof{id} = [dt.hof{id}, featHist(i,34)];
%     dt.mbhx{id} = [dt.mbhx{id}, featHist(i,35)];
%     dt.mbhy{id} = [dt.mbhy{id}, featHist(i,36)];
% end

%% Assign each DT to any touched Segments
fprintf('Assigning DT to Segments ... \n');
for i=1:size(featHist,1)
    x=featHist(i,2:2:31); x(x>I_w)=I_w; x(x<1)=1; % x is the column
    y=featHist(i,3:2:31); y(y>I_h)=I_h; y(y<1)=1; % y is the row
    f=featHist(i,1)-14:featHist(i,1); f(f>frame_num)=frame_num; f(f<1)=1;
    posVect = y+(x-1).*I_h+(f-1).*(I_h*I_w);
    ids = unique(svMap(posVect));
    for k=1:length(ids)
        id = ids(k);
        segments(id).IDX = [segments(id).IDX, i];
        segments(id).traj = [segments(id).traj, featHist(i,32)];
        segments(id).hog = [segments(id).hog, featHist(i,33)];
        segments(id).hof = [segments(id).hof, featHist(i,34)];
        segments(id).mbhx = [segments(id).mbhx, featHist(i,35)];
        segments(id).mbhy = [segments(id).mbhy, featHist(i,36)];
    end
end

%% Sort to histogram
for i=1:segmentNum
    if ~isempty(segments(i).IDX)
        dt.traj(i,:) = hist(segments(i).traj, 1:centerNum.traj)./numel(segments(i).IDX);
        dt.hog(i,:) = hist(segments(i).hog, 1:centerNum.hog)./numel(segments(i).IDX);
        dt.hof(i,:) = hist(segments(i).hof, 1:centerNum.hof)./numel(segments(i).IDX);
        dt.mbhx(i,:) = hist(segments(i).mbhx, 1:centerNum.mbhx)./numel(segments(i).IDX);
        dt.mbhy(i,:) = hist(segments(i).mbhy, 1:centerNum.mbhy)./numel(segments(i).IDX);
    end
end
fv = cat(2, dt.traj, dt.hog, dt.hof, dt.mbhx, dt.mbhy);

%% Save file
savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'fv', '-v7.3');
