function [segLabels, labelList, segSizes] = svGroundTruth(svMap_path, gt_path)
% This function is used to assign Ground-Truth labels to segments.
% 1: background segments
% >1: segments with semantic labels
% 0: unlabeled segments

%% Load Video Segmentation
load(svMap_path); % svMap
svMap = double(svMap);
svNum = length(unique(svMap));
[I_h, I_w, ~] = size(svMap);


%% Load Ground-Truth
gtList = dir(fullfile(gt_path, '*.pgm'));
gtFrameNum = length(gtList);
gtMap = zeros(I_h, I_w, gtFrameNum);
gtFrameID = zeros(gtFrameNum,1);
for i=1:length(gtList)
    gtMap(:,:,i) = imread(fullfile(gt_path, gtList(i).name));
    gtFrameID(i) = str2double(gtList(i).name(1:5));
end
gtMap = double(gtMap);
gtMap(gtMap==0) = 1; % background get assigned ID 1


%% Assign GT Labels to Svx

labelList = unique(gtMap);
segLabels = zeros(svNum,1);
segSizes = zeros(svNum,1);

svMap = svMap(:,:,gtFrameID);
svx_on = unique(svMap);
for i=1:length(svx_on)
    this_pos = (svMap==svx_on(i));
    segSizes(svx_on(i)) = nnz(this_pos);
    svx_labels = gtMap(this_pos);
    segLabels(svx_on(i)) = mode(svx_labels);
end

