function potential = svPotential(lib, model_path, svDescBase_path, svDescDt_path, savepath)
% This function is used to compute segment potential of a video
% classType = 'Obj'/'Act'

%% Load video segment features
fprintf('Loading video segment features ... \n');
base = load(svDescBase_path);
dt = load(svDescDt_path);
fv = cat(2, base.fv, dt.fv);

%% Load Trained Models
mat_list = dir(fullfile(model_path,'*.mat'));
for i=1:length(mat_list)
    models{i} = load(fullfile(model_path, mat_list(i).name));
end

%% Compute Segment Potential
addpath(genpath(lib));
potential = zeros(size(fv,1),length(models));
for i=1:length(models)
    fprintf('Computing potential for model %d ...\n', i);
    if ~isempty(models{i})
        [label, accuracy, probs] = ...
            svmpredict(ones(size(fv,1),1), fv, models{i}.model.model, '-b 1');
        potential(:,i) = probs(:,1);
    end
end

% save the potential matrix
savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'potential', '-v7.3');
