function potential = svPotentialAllSet(lib, LabelSet, model_path, svDescBase_path, svDescDt_path, savepath)
% This function is used to compute segment potential of a video
% classType = 'ObjAct'/'ActOnObj'/'ObjOnAct'

%% Load video segment features
fprintf('Loading video segment features ... \n');
base = load(svDescBase_path);
dt = load(svDescDt_path);
fv = cat(2, base.fv, dt.fv);

%% Load models
fprintf('Loading Models ... \n');
cnt = 0;
for i=1:LabelSet.objNum
    for j=1:LabelSet.actNum
        cnt = cnt + 1;
        thislabel = i*10+j;
        mat_name = sprintf('%02d.mat', thislabel);
        mat_path = fullfile(model_path, mat_name);
        if exist(mat_path, 'file')
            models{cnt} = load(mat_path);
        else
            models{cnt} = [];
        end
    end
end

%% Compute Segment Potential
addpath(genpath(lib));
potential = zeros(size(fv,1),length(models));
for i=1:length(models)
    fprintf('Computing potential for model %d ...\n', i);
    if ~isempty(models{i})
        [label, accuracy, probs] = ...
            svmpredict(ones(size(fv,1),1), fv, models{i}.model.model, '-b 1');
        potential(:,i) = probs(:,1);
    end
end

% save the potential matrix
savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'potential', '-v7.3');
