function potential = svPotentialBK(lib, PATH, LabelSet, classType, svDescBase_path, svDescDt_path, savepath)
% This function is used to compute segment potential of a video

%% Load video segment features
fprintf('Loading video segment features ... \n');
base = load(svDescBase_path);
dt = load(svDescDt_path);
fv = cat(2, base.fv, dt.fv);


%% Load models
fprintf('Loading Models ... \n');
models{1} = load(fullfile(PATH.svTSPClsf, classType, 'BK.mat'));

%% Compute Segment Potential
addpath(genpath(lib));
potential = zeros(size(fv,1),length(models));
for i=1:length(models)
    fprintf('Computing potential for model %d ...\n', i);
    [label, accuracy, probs] = ...
        svmpredict(ones(size(fv,1),1), fv, models{i}.model.model, '-b 1');
    potential(:,i) = probs(:,1);
end

% save the potential matrix
savefold = fileparts(savepath);
if ~exist(savefold, 'dir')
    mkdir(savefold);
end
save(savepath, 'potential', '-v7.3');
